/**
 * Autogenerated by Thrift Compiler (0.14.0)
 *
 * DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
 *  @generated
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using Thrift;
using Thrift.Collections;
using System.ServiceModel;
using System.Runtime.Serialization;

using Thrift.Protocol;
using Thrift.Protocol.Entities;
using Thrift.Protocol.Utilities;
using Thrift.Transport;
using Thrift.Transport.Client;
using Thrift.Transport.Server;
using Thrift.Processor;


#pragma warning disable IDE0079  // remove unnecessary pragmas
#pragma warning disable IDE1006  // parts of the code use IDL spelling

namespace ThriftTest
{

[DataContract(Namespace="")]
public partial class OptionalSetDefaultTest : TBase
{
  private THashSet<string> _with_default;

  [DataMember(Order = 0)]
  public THashSet<string> With_default
  {
    get
    {
      return _with_default;
    }
    set
    {
      __isset.with_default = true;
      this._with_default = value;
    }
  }


  [DataMember(Order = 1)]
  public Isset __isset;
  [DataContract]
  public struct Isset
  {
    [DataMember]
    public bool with_default;
  }

  #region XmlSerializer support

  public bool ShouldSerializeWith_default()
  {
    return __isset.with_default;
  }

  #endregion XmlSerializer support

  public OptionalSetDefaultTest()
  {
    this._with_default = new THashSet<string>();
    this._with_default.Add("test");
    this.__isset.with_default = true;
  }

  public OptionalSetDefaultTest DeepCopy()
  {
    var tmp199 = new OptionalSetDefaultTest();
    if((With_default != null) && __isset.with_default)
    {
      tmp199.With_default = this.With_default.DeepCopy();
    }
    tmp199.__isset.with_default = this.__isset.with_default;
    return tmp199;
  }

  public async global::System.Threading.Tasks.Task ReadAsync(TProtocol iprot, CancellationToken cancellationToken)
  {
    iprot.IncrementRecursionDepth();
    try
    {
      TField field;
      await iprot.ReadStructBeginAsync(cancellationToken);
      while (true)
      {
        field = await iprot.ReadFieldBeginAsync(cancellationToken);
        if (field.Type == TType.Stop)
        {
          break;
        }

        switch (field.ID)
        {
          case 1:
            if (field.Type == TType.Set)
            {
              {
                TSet _set200 = await iprot.ReadSetBeginAsync(cancellationToken);
                With_default = new THashSet<string>(_set200.Count);
                for(int _i201 = 0; _i201 < _set200.Count; ++_i201)
                {
                  string _elem202;
                  _elem202 = await iprot.ReadStringAsync(cancellationToken);
                  With_default.Add(_elem202);
                }
                await iprot.ReadSetEndAsync(cancellationToken);
              }
            }
            else
            {
              await TProtocolUtil.SkipAsync(iprot, field.Type, cancellationToken);
            }
            break;
          default: 
            await TProtocolUtil.SkipAsync(iprot, field.Type, cancellationToken);
            break;
        }

        await iprot.ReadFieldEndAsync(cancellationToken);
      }

      await iprot.ReadStructEndAsync(cancellationToken);
    }
    finally
    {
      iprot.DecrementRecursionDepth();
    }
  }

  public async global::System.Threading.Tasks.Task WriteAsync(TProtocol oprot, CancellationToken cancellationToken)
  {
    oprot.IncrementRecursionDepth();
    try
    {
      var struc = new TStruct("OptionalSetDefaultTest");
      await oprot.WriteStructBeginAsync(struc, cancellationToken);
      var field = new TField();
      if((With_default != null) && __isset.with_default)
      {
        field.Name = "with_default";
        field.Type = TType.Set;
        field.ID = 1;
        await oprot.WriteFieldBeginAsync(field, cancellationToken);
        {
          await oprot.WriteSetBeginAsync(new TSet(TType.String, With_default.Count), cancellationToken);
          foreach (string _iter203 in With_default)
          {
            await oprot.WriteStringAsync(_iter203, cancellationToken);
          }
          await oprot.WriteSetEndAsync(cancellationToken);
        }
        await oprot.WriteFieldEndAsync(cancellationToken);
      }
      await oprot.WriteFieldStopAsync(cancellationToken);
      await oprot.WriteStructEndAsync(cancellationToken);
    }
    finally
    {
      oprot.DecrementRecursionDepth();
    }
  }

  public override bool Equals(object that)
  {
    if (!(that is OptionalSetDefaultTest other)) return false;
    if (ReferenceEquals(this, other)) return true;
    return ((__isset.with_default == other.__isset.with_default) && ((!__isset.with_default) || (TCollections.Equals(With_default, other.With_default))));
  }

  public override int GetHashCode() {
    int hashcode = 157;
    unchecked {
      if((With_default != null) && __isset.with_default)
      {
        hashcode = (hashcode * 397) + TCollections.GetHashCode(With_default);
      }
    }
    return hashcode;
  }

  public override string ToString()
  {
    var sb = new StringBuilder("OptionalSetDefaultTest(");
    int tmp204 = 0;
    if((With_default != null) && __isset.with_default)
    {
      if(0 < tmp204++) { sb.Append(", "); }
      sb.Append("With_default: ");
      With_default.ToString(sb);
    }
    sb.Append(')');
    return sb.ToString();
  }
}

}
