/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

// This is an automatically generated file.

#ifndef YARP_ROSMSG_deprecated1_diagnostic_msgs_DiagnosticArray_h
#define YARP_ROSMSG_deprecated1_diagnostic_msgs_DiagnosticArray_h

#include <yarp/conf/system.h>

YARP_COMPILER_DEPRECATED_WARNING(<diagnostic_msgs_DiagnosticArray.h> header is deprecated. Use <yarp/rosmsg/diagnostic_msgs/DiagnosticArray.h> instead)

#include <yarp/rosmsg/diagnostic_msgs/DiagnosticArray.h>
#include <std_msgs_Header.h>
#include <diagnostic_msgs_DiagnosticStatus.h>
#include <yarp/conf/api.h>

YARP_DEPRECATED_MSG("diagnostic_msgs_DiagnosticArray is deprecated, use yarp::rosmsg::diagnostic_msgs::DiagnosticArray instead")
typedef yarp::rosmsg::diagnostic_msgs::DiagnosticArray diagnostic_msgs_DiagnosticArray;

#endif // YARP_ROSMSG_deprecated1_diagnostic_msgs_DiagnosticArray_h
